﻿Module Module1

    Class fileobj
        Implements IEquatable(Of fileobj)

        Public Overrides Function GetHashCode() As Integer
            Return 0
        End Function

        Public Overloads Function Equals(other As fileobj) As Boolean Implements System.IEquatable(Of fileobj).Equals

            Return Me.size = other.size AndAlso
                Me.middlebytes = other.middlebytes AndAlso
                Me.md5.SequenceEqual(other.md5)

        End Function

        Property name As String
        Sub New(filename As String)
            Me.name = filename
        End Sub

        Private _size As Long
        Private havesize As Boolean = False
        Public ReadOnly Property size() As Long
            Get
                If Not havesize Then
                    Me._size = FileIO.FileSystem.GetFileInfo(Me.name).Length
                    havesize = True
                End If
                Return _size
            End Get
        End Property

        Private _middlebytes As UInteger
        Private havemiddlebytes As Boolean = False
        Public ReadOnly Property middlebytes() As UInteger
            Get
                If Not havemiddlebytes Then
                    Try
                        Dim fs As New IO.FileStream(Me.name, IO.FileMode.Open, IO.FileAccess.Read)
                        Dim buf(3) As Byte
                        If Me._size >= 4 Then
                            fs.Seek(Me._size / 2, IO.SeekOrigin.Begin)
                            fs.Read(buf, 0, 4)
                        Else
                            fs.Read(buf, 0, Me._size)
                        End If
                        fs.Close()

                        _middlebytes = BitConverter.ToUInt32(buf, 0)

                        Me.havemiddlebytes = True
                    Catch ex As Exception
                        Console.WriteLine("Couldn't open file {0}", Me.name)
                        Console.WriteLine(ex.ToString)
                    End Try
                End If
                Return _middlebytes
            End Get
        End Property

        Private _md5 As Byte()
        Private havemd5 As Boolean = False
        Public ReadOnly Property md5() As Byte()
            Get
                If Not havemd5 Then
                    Dim hashr As System.Security.Cryptography.MD5 = System.Security.Cryptography.MD5.Create
                    Dim fs As New IO.FileStream(Me.name, IO.FileMode.Open, IO.FileAccess.Read, IO.FileShare.Read)
                    hashr.ComputeHash(fs)
                    _md5 = hashr.Hash
                    fs.Close()
                End If
                Return _md5
            End Get
        End Property

    End Class

    MustInherit Class rule

        Enum RuleAction
            Ignore
            Keep
            Delete
        End Enum

        Public action As RuleAction
        Public text As String


        Shared Function parse(def As String) As rule
            Dim newrule As rule = Nothing

            Dim splitrule As String() = def.ToLowerInvariant.Split(" ")
            If splitrule.Length < 2 Then
                Throw New Exception("An action and type are both required")
            End If

            Select Case splitrule(1)
                Case "regex"
                    newrule = New regexrule(def)
                Case Else
                    Throw New Exception("Unknown filter type """ & splitrule(1) & """")
            End Select

            Select Case splitrule(0)
                Case "ignore"
                    newrule.action = RuleAction.Ignore
                Case "keep"
                    newrule.action = RuleAction.Keep
                Case "delete"
                    newrule.action = RuleAction.Delete
                Case Else
                    Throw New Exception("Unknown action type """ & splitrule(0) & """")
            End Select

            Return newrule
        End Function

        MustOverride Function filtergroup(g As IEnumerable(Of fileobj)) As List(Of fileobj)

    End Class

    Class regexrule
        Inherits rule

        Public Sub New(def As String)

        End Sub

        Public Overrides Function filtergroup(g As IEnumerable(Of fileobj)) As List(Of fileobj)

        End Function

    End Class

    Dim allfilelist As New List(Of fileobj)
    Dim rulelist As New List(Of rule)

    Sub Main()
        Dim dir As String = Environment.CurrentDirectory
        Console.WriteLine("Enumerating files...")
        For Each f In FileIO.FileSystem.GetFiles(dir, FileIO.SearchOption.SearchAllSubDirectories).ToList
            allfilelist.Add(New fileobj(f))
        Next

        Console.WriteLine("Total files: {0:n0}", allfilelist.Count)
        Console.WriteLine("Checking for duplicates...")

        Dim dupegroups = From file In allfilelist Group By file Into Group Where Group.Count > 1

        Console.WriteLine("Duplicates: {0:n0}", dupegroups.Count)

        While True

        End While

        Console.WriteLine()
        Console.WriteLine("Press enter to exit")
        Console.ReadLine()
    End Sub


End Module
